//------------------------------------------------------------
//
//	Original C++ code by Leyder Dylan http://www.slug-production.be.tf/
//
//	File : "MD2.c"
//
//	Converted to C by John
//
//------------------------------------------------------------

#include "WindowGL.h"
#include "MD2.h"

BOOL TEXTURE_LoadTGA(TEXTURE * pTxt, char* filename, float minFilter, float maxFilter)
{
	GLubyte		TGAheader[12] = {0,0,2,0,0,0,0,0,0,0,0,0};
	GLubyte		TGAcompare[12];
	GLubyte		header[6];
	GLuint		bytesPerPixel;
	GLuint		imageSize;
	GLuint		temp;
	GLuint		type=GL_RGBA;

	FILE* file = fopen(filename, "rb");

	if(file==NULL || fread(TGAcompare, 1, sizeof(TGAcompare), file) != sizeof(TGAcompare) ||
		    memcmp(TGAheader, TGAcompare, sizeof(TGAheader))!= 0 ||
		    fread(header, 1, sizeof(header), file) != sizeof(header)){
		if(file == NULL)
		{
			MessageBox(NULL, "TGA : File not found!", "MD2 : Error ...", MB_ICONERROR);
			return GL_FAIL;
		}else{
			fclose(file);
			MessageBox(NULL,"TGA : File corrupt!", "MD2 : Error ...", MB_ICONERROR);
			return GL_FAIL;
		}
	}

	pTxt->width = header[1] * 256 + header[0];
	pTxt->height= header[3] * 256 + header[2];

	if(pTxt->width <= 0 || pTxt->height <= 0 || (header[4] != 24 && header[4] != 32))
	{
		fclose(file);
		MessageBox(NULL,"TGA : Not 24 or 32 bpp!", "MD2 : Error ...", MB_ICONERROR);
		return GL_FAIL;
	}

	pTxt->bpp	= header[4];
	bytesPerPixel= pTxt->bpp / 8;
	imageSize	 = pTxt->width * pTxt->height * bytesPerPixel;

	GLubyte * data = malloc(imageSize);

	if(data == NULL || fread(data, 1, imageSize, file) != imageSize){
		if(data != NULL)
			free(data);

		MessageBox(NULL,"TGA : Report storage!", "MD2 : Error ...", MB_ICONERROR);

		fclose(file);
		return GL_FAIL;
	}

	for(GLuint i=0; i<imageSize; i += bytesPerPixel){
		temp	  = data[i];
		data[i]	  = data[i + 2];
		data[i+2] = temp;
	}

	fclose (file);


	glGenTextures(1, &pTxt->ID);

	glBindTexture(GL_TEXTURE_2D, pTxt->ID);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, minFilter);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, maxFilter);

	if(pTxt->bpp==24)
		type=GL_RGB;

	glTexImage2D(GL_TEXTURE_2D, 0, type, pTxt->width, pTxt->height, 0, type, GL_UNSIGNED_BYTE, data);

	return GL_OK;
}


// **********************************************

void MD2_Init(MD2 * pMD2)
{
	pMD2->stateStart 	= IDLE1_START;
	pMD2->stateEnd   	= IDLE1_END;
	pMD2->numGlCommands = 0;
	pMD2->frameSize  	= 0;
	pMD2->numFrames  	= 0;
	pMD2->currentFrame	= IDLE1_START;
	pMD2->nextFrame		= pMD2->currentFrame + 1;
	pMD2->endFrame		= IDLE1_END;
	pMD2->interpolation	= 0.0f;
}

BOOL MD2_Load(MD2 * pMD2, char* filename)
{
	FILE* file;
	MD2_HEADER header;

	MD2_Init(pMD2);

	if(!(file = fopen(filename, "rb"))){
		MessageBox(NULL,"MD2 : File not found!", "MD2 : Error ...", MB_ICONERROR);
		return GL_FAIL;
	}

	fread(&header, sizeof(MD2_HEADER), 1, file);

	if(header.magic != 844121161){
		MessageBox(NULL,"MD2 : Invalide Header!", "MD2 : Error ...", MB_ICONERROR);
		return GL_FAIL;
	}

	if(header.version != 8){
		MessageBox(NULL,"MD2 : Invalide Header!", "MD2 : Error ...", MB_ICONERROR);
		return GL_FAIL;
	}

	pMD2->frames = malloc(header.frameSize * header.numFrames);

	if(pMD2->frames == NULL)
		return GL_FAIL;

	fseek(file, header.offsetFrames, SEEK_SET);
	fread(pMD2->frames, header.frameSize * header.numFrames, 1, file);

	pMD2->glCommands = malloc(header.numGlCommands * sizeof(long));

	if(pMD2->glCommands == NULL)
		return GL_FAIL;

	fseek(file, header.offsetGlCommands, SEEK_SET);

	fread(pMD2->glCommands, header.numGlCommands * sizeof(long), 1, file);

	pMD2->numFrames	    = header.numFrames;
	pMD2->numGlCommands = header.numGlCommands;
	pMD2->frameSize	    = header.frameSize;
	pMD2->numTriangles  = header.numTriangles;

	fclose(file);

	return GL_OK;
}


// **********************************************
void MD2_Render(MD2 * pMD2, int numFrame, int cut)
{
	static MD2_MODELVERTEX vertList[100];

	MD2_FRAME_PTR	pCurrentFrame;
	float 			v1[3];
	long *			command;
	float			texcoord[2];
	int				loop;
	int				vertIndex;
	int				type;
	int				numVertex;
	int				index;

	pCurrentFrame = (MD2_FRAME*) ((char*)(pMD2->frames)+pMD2->frameSize*numFrame);
	command		  = pMD2->glCommands;

	while((*command)!=0){
		if(*command>0){
			numVertex = *command;
			command++;
			type= 0;
		}else{
			numVertex = - *command;
			command++;
			type= 1;
		}

		if(numVertex<0)
			numVertex = -numVertex;

		index = 0;

		for(loop=0; loop<numVertex; loop++){
			vertList[index].u = *((float*)command);
			command++;
			vertList[index].v = *((float*)command);
			command++;

			vertIndex = *command;
			command++;

			vertList[index].x = ( (pCurrentFrame->vertices[vertIndex].vertex[0]*
				    pCurrentFrame->scale[0])+ pCurrentFrame->translate[0]);

			vertList[index].z = -((pCurrentFrame->vertices[vertIndex].vertex[1]*
				    pCurrentFrame->scale[1])+ pCurrentFrame->translate[1]);

			vertList[index].y = ( (pCurrentFrame->vertices[vertIndex].vertex[2]*
				    pCurrentFrame->scale[2])+ pCurrentFrame->translate[2]);

			index++;
		}

		if(type == 0){
			glBegin(GL_TRIANGLE_STRIP);
			for(loop=0; loop<index; loop++)	{
				v1[0] = (vertList[loop].x / cut);
				v1[1] = (vertList[loop].y / cut);
				v1[2] = (vertList[loop].z / cut);

				texcoord[0] = vertList[loop].u;
				texcoord[1] = vertList[loop].v;

				glTexCoord2fv(texcoord);
				glVertex3fv(v1);
			}
			glEnd();
		}else{
			glBegin(GL_TRIANGLE_FAN);
			for(loop=0; loop<index; loop++)	{
				v1[0] = (vertList[loop].x / cut);
				v1[1] = (vertList[loop].y / cut);
				v1[2] = (vertList[loop].z / cut);

				texcoord[0] = vertList[loop].u;
				texcoord[1] = vertList[loop].v;

				glTexCoord2fv(texcoord);
				glVertex3fv(v1);
			}
			glEnd();
		}
	}
}


// **********************************************
void MD2_Animate(MD2 * pMD2, int startFrame, int EndFrame, float percent, int cut)
{

	if(pMD2->currentFrame == -1 || pMD2->currentFrame > pMD2->endFrame){
		pMD2->currentFrame = startFrame;
		pMD2->nextFrame	   = pMD2->currentFrame+1;
		pMD2->endFrame	   = EndFrame;
	}

	if(pMD2->interpolation > 1.0f){
		pMD2->interpolation = 0.0f;
		pMD2->currentFrame  = pMD2->nextFrame;
		pMD2->nextFrame	    = pMD2->currentFrame+1;
	}

	pMD2->interpolation += percent;
	MD2_Render(pMD2, pMD2->currentFrame, cut);

}

void MD2_SetState(MD2 * pMD2, int state)
{
	switch(state){
	case IDLE1:
		pMD2->stateStart   = IDLE1_START;
		pMD2->stateEnd	   = IDLE1_END;
		pMD2->currentFrame = -1;
		break;
	case RUN:
		pMD2->stateStart   = RUN_START;
		pMD2->stateEnd	   = RUN_END;
		pMD2->currentFrame = -1;
		break;
	case SHOT_STAND:
		pMD2->stateStart   = SHOT_STAND_START;
		pMD2->stateEnd	   = SHOT_STAND_END;
		pMD2->currentFrame = -1;
		break;
/*
	case SHOT_SHOULDER:
		pMD2->stateStart   = SHOT_SHOULDER_START;
		pMD2->stateEnd	   = SHOT_SHOULDER_END;
		pMD2->currentFrame = -1;
		break;
	case JUMP:
		pMD2->stateStart   = JUMP_START;
		pMD2->stateEnd	   = JUMP_END;
		pMD2->currentFrame = -1;
		break;
	case IDLE2:
		pMD2->stateStart   = IDLE2_START;
		pMD2->stateEnd	   = IDLE2_END;
		pMD2->currentFrame = -1;
		break;
	case SHOT_FALLDOWN:
		pMD2->stateStart   = SHOT_FALLDOWN_START;
		pMD2->stateEnd	   = SHOT_FALLDOWN_END;
		pMD2->currentFrame = -1;
		break;
	case IDLE3:
		pMD2->stateStart   = IDLE3_START;
		pMD2->stateEnd	   = IDLE3_END;
		pMD2->currentFrame = -1;
		break;
	case IDLE4:
		pMD2->stateStart   = IDLE4_START;
		pMD2->stateEnd	   = IDLE4_END;
		pMD2->currentFrame = -1;
		break;
	case CROUCH:
		pMD2->stateStart   = CROUCH_START;
		pMD2->stateEnd	   = CROUCH_END;
		pMD2->currentFrame = -1;
		break;
	case CROUCH_CRAWL:
		pMD2->stateStart   = CROUCH_CRAWL_START;
		pMD2->stateEnd	   = CROUCH_CRAWL_END;
		pMD2->currentFrame = -1;
		break;
	case CROUCH_IDLE:
		pMD2->stateStart   = CROUCH_IDLE_START;
		pMD2->stateEnd	   = CROUCH_IDLE_END;
		pMD2->currentFrame = -1;
		break;
	case CROUCH_DEATH:
		pMD2->stateStart   = CROUCH_DEATH_START;
		pMD2->stateEnd	   = CROUCH_DEATH_END;
		pMD2->currentFrame = -1;
		break;
	case DEATH_FALLBACK:
		pMD2->stateStart   = DEATH_FALLBACK_START;
		pMD2->stateEnd	   = DEATH_FALLBACK_END;
		pMD2->currentFrame = -1;
		break;
	case DEATH_FALLFORWARD:
		pMD2->stateStart   = DEATH_FALLFORWARD_START;
		pMD2->stateEnd	   = DEATH_FALLFORWARD_END;
		pMD2->currentFrame = -1;
		break;
	case DEATH_FALLBACKSLOW:
		pMD2->stateStart   = DEATH_FALLBACKSLOW_START;
		pMD2->stateEnd	   = DEATH_FALLBACKSLOW_END;
		pMD2->currentFrame = -1;
		break;
*/
	}
}


