//------------------------------------------------------------
//
//	Original C++ code by Leyder Dylan http://www.slug-production.be.tf/
//
//	File : "MD2.h"
//
//	Converted to C by John
//
//	Libs to link for lcc-win32: opengl32.lib glu32.lib
//
//------------------------------------------------------------

#include <windows.h>
#include <gl\gl.h>
#include <gl\glu.h>

#include <stdio.h>


// MD2 Constants
#define MAX_TRIANGLES 4096
#define MAX_VERTICES  2048
#define MAX_TEXCOORDS 2048
#define MAX_FRAMES    512

#define CHUNK1 0
#define CHUNK2 1
#define CHUNK3 2
#define CHUNK4 3

// MD2 State Constants
#define ANIMATE_CURRENT -100

// Three anim sequences are contained within the MD2 file.

#define IDLE1				0
#define RUN					1
#define SHOT_STAND			2

/*
 The original md2 file also had these extra animation sequences.

 I took them out to save on size of files.

#define SHOT_SHOULDER		3
#define JUMP				4
#define IDLE2				5
#define SHOT_FALLDOWN		6
#define IDLE3				7
#define IDLE4				8
#define CROUCH				9
#define CROUCH_CRAWL		10
#define CROUCH_IDLE			11
#define CROUCH_DEATH		12
#define DEATH_FALLBACK		13
#define DEATH_FALLFORWARD	14
#define DEATH_FALLBACKSLOW	15
*/

#define IDLE1_START					0
#define IDLE1_END					39
#define RUN_START					40
#define RUN_END						45
#define SHOT_STAND_START			46
#define SHOT_STAND_END				61


/*
 The original md2 file also had these extra animation sequences.

 I took them out to save on size of files.

#define SHOT_SHOULDER_START			62
#define SHOT_SHOULDER_END			66
#define JUMP_START					67
#define JUMP_END					73
#define IDLE2_START					74
#define IDLE2_END					95
#define SHOT_FALLDOWN_START			96
#define SHOT_FALLDOWN_END			112
#define IDLE3_START					113
#define IDLE3_END					122
#define IDLE4_START					123
#define IDLE4_END					135
#define CROUCH_START				136
#define CROUCH_END					154
#define CROUCH_CRAWL_START			155
#define CROUCH_CRAWL_END			161
#define CROUCH_IDLE_START			162
#define CROUCH_IDLE_END				169
#define CROUCH_DEATH_START			170
#define CROUCH_DEATH_END			177
#define DEATH_FALLBACK_START		178
#define DEATH_FALLBACK_END			185
#define DEATH_FALLFORWARD_START		186
#define DEATH_FALLFORWARD_END		190
#define DEATH_FALLBACKSLOW_START	191
#define DEATH_FALLBACKSLOW_END		198
*/

typedef struct MD2_HEADER_TYP
{
	int magic;
	int version;
	int skinWidth;
	int skinHeight;
	int frameSize;
	int numSkins;
	int numVertices;
	int numTexcoords;
	int numTriangles;
	int numGlCommands;
	int numFrames;
	int offsetSkins;
	int offsetTexcoords;
	int offsetTriangles;
	int offsetFrames;
	int offsetGlCommands;
	int offsetEnd;
} MD2_HEADER, *MD2_HEADER_PTR;

typedef struct MD2_VERTEX_TYP
{
	unsigned char vertex[3];
	unsigned char lightNormalIndex;
} MD2_VERTEX, *MD2_VERTEX_PTR;

typedef struct MD2_FRAME_TYP
{
	float	   scale[3];
	float	   translate[3];
	char	   name[16];
	MD2_VERTEX vertices[1];
} MD2_FRAME, *MD2_FRAME_PTR;

typedef struct MD2_MODELVERTEX_TYP
{
	float x,y,z;
	float u,v;
} MD2_MODELVERTEX, *MD2_MODELVERTEX_PTR;

typedef struct _TEXTURE
{
	GLubyte* data;
	unsigned int bpp;
	unsigned int width;
	unsigned int height;
	unsigned int ID;
}TEXTURE;

BOOL TEXTURE_LoadTGA(TEXTURE * pTxt, char* filename, float minFilter, float maxFilter);

typedef struct _MD2
{
	int		numGlCommands;
	long *  glCommands;
	int 	numTriangles;
	int 	stateStart;
	int 	stateEnd;
	int 	frameSize;
	int 	numFrames;
	char *  frames;
	int 	currentFrame;
	int 	nextFrame;
	int 	endFrame;
	float 	interpolation;
}MD2;

BOOL MD2_Load(MD2 * pMD2, char* filename);
void MD2_Render(MD2 * pMD2, int numFrame, int cut);
void MD2_Animate(MD2 * pMD2, int startFrame, int EndFrame, float Interpolation, int cut);
void MD2_SetState(MD2 * pMD2, int state);
void MD2_Init(MD2 * pMD2);

