//------------------------------------------------------------
//
//	Original C++ code by Leyder Dylan http://www.slug-production.be.tf/
//
//	File : "Main.c"
//
//	Converted to C by John
//
//	Libs to link for lcc-win32: opengl32.lib glu32.lib
//
//
// I wrote to Leyder Dylan asking if he minded me using he stuff
// for my web page, he didn't answer!
//
//------------------------------------------------------------

// ESC key pressed- exit
// 'I' key pressed - the 'IDLE1' animation
// 'R' key pressed - the 'RUN' animation
// 'S' key pressed - the 'SHOT_STAND' animation

#include <windows.h>
#include "WindowGL.h"
#include "MD2.h"

// *********************************************
// Variables used for creating the OpenGL window
// *********************************************
int Resolution_X = 640;		// Resolution on X
int Resolution_Y = 480;		// Resolution on Y

int Bits = 16;				// Bpp

extern BOOL bFullScreen;	// FullScreen or not ?
extern BOOL bActive;		// Is the window active ?
extern HDC hDC;				// Window Handle

char * Title = "MD2 Quake 2 Model - Programmed by : Leyder Dylan (http://www.slug-production.be.tf)";	// Title

// For using the keyboard keys
extern BOOL keys[];

// For the the Model
MD2 Md2_Model;
MD2 Md2_Weapon;

// For the Model Texture
TEXTURE Md2_Skin_Texture;
TEXTURE Md2_Weapon_Texture;

// **********************************************
int InitGL(GLvoid) // The OpenGL initialisation
{
	// ********** The MD2 **********
	// The texture
	if(GL_OK != TEXTURE_LoadTGA(&Md2_Skin_Texture, "Data/Goblin.tga", GL_LINEAR, GL_LINEAR))
		return GL_FAIL;;

	// The MD2 Goblin.md2
	if(GL_OK != MD2_Load(&Md2_Model, "Data/Goblin1.md2"))
		return GL_FAIL;;

	// *****************************

	// ********** The Weapon *******
	// The Texture
	if (GL_OK != TEXTURE_LoadTGA(&Md2_Weapon_Texture, "Data/Weapon.tga", GL_LINEAR, GL_LINEAR))
		return GL_FAIL;;

	// MD2 OK
	if(GL_OK != MD2_Load(&Md2_Weapon, "Data/Weapon1.md2"))
		return GL_FAIL;;

	// We enable the texture
	glEnable(GL_TEXTURE_2D);

	// We enable the depth test
	glEnable(GL_DEPTH_TEST);

	// The Model Animation		==> See in "MD2.h" for the others animations
	MD2_SetState(&Md2_Model, IDLE1);

	// The Weapon Animation		==> See in "MD2.h" for the others animations
	MD2_SetState(&Md2_Weapon, IDLE1);

	return TRUE;
}


// **********************************************
int DrawGLScene(GLvoid)	// Draw our OpenGL scene
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// Clean the screen and the depth buffer
	glLoadIdentity();									// Reset The Projection Matrix

	// We move the camera
	gluLookAt( 0, 0, 6, 0, 0, 0, 0, 1, 0);

	// The Skin Texture
	glBindTexture(GL_TEXTURE_2D, Md2_Skin_Texture.ID);

	/*
	Params for : "Md2_Model.Animate(Md2_Model.stateStart, Md2_Model.stateEnd, 0.1f,20)"

	Md2_Model.stateStart  = First image of the animation
	Md2_Model.stateEnd	  = Last image of the animation
	0.1f = Animation Speed in %
	20   = MS2 Size

	*/
	// The Model and his animation
	MD2_Animate(&Md2_Model, Md2_Model.stateStart, Md2_Model.stateEnd, 0.1f, 20);

	// The Weapon Skin
	glBindTexture(GL_TEXTURE_2D, Md2_Weapon_Texture.ID);

	/*
	Params for : "Md2_Weapon.Animate(Md2_Weapon.stateStart, Md2_Weapon.stateEnd, 0.1f,20);

	Md2_Weapon.stateStart  = First image of the animation
	Md2_Weapon.stateEnd	   = Last image of the animation
	0.1f = Animation Speed in %
	20   = MS2 Size

	*/

	// The Weapon and his animation
	MD2_Animate(&Md2_Weapon, Md2_Weapon.stateStart, Md2_Weapon.stateEnd, 0.1f, 20);

	return TRUE;

}


// **********************************************
void Program_Exit()
{
	// release any memory that was allocated
	if(Md2_Model.frames) 		free(Md2_Model.frames);
	if(Md2_Weapon.frames) 		free(Md2_Weapon.frames);
	if(Md2_Model.glCommands) 	free(Md2_Model.glCommands);
	if(Md2_Weapon.glCommands) 	free(Md2_Weapon.glCommands);
	if(Md2_Skin_Texture.data) 	free(Md2_Skin_Texture.data);
	if(Md2_Weapon_Texture.data) free(Md2_Weapon_Texture.data);

	// We kill the window
	KillGLWindow();
}

// **********************************************
int WINAPI WinMain(	HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	MSG		msg;									// Windows Message Structure
	BOOL	done = FALSE;							// Bool Variable To Exit Loop

	// FullScreen or not ?
//	if (MessageBox(NULL, "Would You Like To Run In Fullscreen Mode?", "Start FullScreen ?", MB_YESNO | MB_ICONQUESTION) == IDNO)
//	{
		bFullScreen = FALSE;		// Windowed mode
//	}

	// Create Our OpenGL Window
	if (!CreateGLWindow(Title, Resolution_X, Resolution_Y, Bits, bFullScreen))
	{
		return 0;	// An error ?
	}

	while(!done)		// Loop That Runs While done = FALSE

	{
		// Is There A Message Waiting ?
		if (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))	// Is There A Message Waiting ?
		{
			if (msg.message == WM_QUIT)			// Have We Received A Quit Message ?
			{
				done = TRUE;				// Yes, done = TRUE, end of the program
			}
			else								// Otherwise, we deal With Window Messages
			{
				TranslateMessage(&msg);		// Translate The Message
				DispatchMessage(&msg);		// Dispatch The Message
			}
		}
		else		// If no messages
		{
			// Draw The Scene.  Watch For ESC Key And Quit Messages From DrawGLScene()
			if (bActive && !DrawGLScene())	// Program active ?
			{
				done = TRUE;
			}
			else
			{
				// Swap the buffers
				SwapBuffers(hDC);

				// ESC key pressed, done = TRUE (end of the loop, end of the program)
				if (keys[VK_ESCAPE]) done = TRUE;

				// If 'I' key pressed, the 'IDLE1' animation
				if (keys['I']){
					MD2_SetState(&Md2_Model, IDLE1);
					MD2_SetState(&Md2_Weapon, IDLE1);
				}
				// If 'R' key pressed, the 'RUN' animation
				if (keys['R']){
					MD2_SetState(&Md2_Model, RUN);
					MD2_SetState(&Md2_Weapon, RUN);
				}
				// If 'J' key pressed, the 'JUMP' animation
				if (keys['S']){
					MD2_SetState(&Md2_Model, SHOT_STAND);
					MD2_SetState(&Md2_Weapon, SHOT_STAND);
				}
			}
		}
	}

	// Shutdown
	Program_Exit();

	return (msg.wParam);

}

