//------------------------------------------------------------
//
//	Original C++ code by Leyder Dylan http://www.slug-production.be.tf/
//
//	File : "WindowGL.c"
//
//	Converted to C by John
//
//	Libs to link for lcc-win32: opengl32.lib glu32.lib
//
//------------------------------------------------------------

#include <windows.h>

#include <gl\gl.h>
#include <gl\glu.h>

#include "WindowGL.h"

HDC			hDC = NULL;		// Private GDI Device Context
HGLRC		hRC = NULL;		// Permanent Rendering Context
HWND		hWnd = NULL;	// Holds Our Window Handle
HINSTANCE	hInstance;		// Holds The Instance Of The Application

BOOL bActive = TRUE;		// Window bActive Flag Set To 'TRUE' By Default
BOOL bFullScreen = TRUE;	// Fullscreen Flag Set To Fullscreen Mode By Default
BOOL keys[256];				// Array Used For The Keyboard Routine


// ***********************************************************************
GLvoid ReSizeGLScene(GLsizei width, GLsizei height)		// Resize And Initialize The GL Window
{
	// Prevent A Divide By Zero By making Height Equal One
	if (height == 0) height = 1;

	glViewport(0, 0, width, height);		// Reset The Current Viewport

	glMatrixMode(GL_PROJECTION);			// Select The Projection Matrix

	glLoadIdentity();						// Select The Projection Matrix

	// Calculate The Aspect Ratio Of The Window
	gluPerspective(45.0f,(float) width / (float) height, 0.1f, 5000.0f);

	glMatrixMode(GL_MODELVIEW);				// Select The Modelview Matrix

	glLoadIdentity();						// Reset The Modelview Matrix

}


// ***********************************************************************
GLvoid KillGLWindow(GLvoid)						// Properly Kill The Window
{
	// Are We In Fullscreen Mode ?
	if (bFullScreen)
	{
		ChangeDisplaySettings(NULL, 0);		// If So Switch Back To The Desktop
		ShowCursor(TRUE);					// Show Mouse Pointer
	}

	// Do We Have A Rendering Context ?
	if (hRC)
	{
		// Are We Able To Release The DC And RC Contexts ?
		if (!wglMakeCurrent(NULL, NULL))
			MessageBox(NULL, "Release Of DC And RC Failed.", "SHUTDOWN ERROR", MB_OK | MB_ICONINFORMATION);

		// Are We Able To Delete The RC ?
		if (!wglDeleteContext(hRC))
			MessageBox(NULL, "Release Rendering Context Failed.", "SHUTDOWN ERROR", MB_OK | MB_ICONINFORMATION);

		// Set RC To NULL
		hRC = NULL;
	}

	// Are We Able To Release The DC
	if (hDC && !ReleaseDC(hWnd, hDC))
	{
		MessageBox(NULL, "Release Device Context Failed.", "SHUTDOWN ERROR", MB_OK | MB_ICONINFORMATION);

		// Set DC To NULL
		hDC = NULL;
	}

	// Are We Able To Destroy The Window ?
	if (hWnd && !DestroyWindow(hWnd))
	{
		MessageBox(NULL, "Could Not Release hWnd.", "SHUTDOWN ERROR", MB_OK | MB_ICONINFORMATION);

		// Set hWnd To NULL
		hWnd = NULL;
	}

	// Are We Able To Unregister Class
	if (!UnregisterClass("OpenGL", hInstance))
	{
		MessageBox(NULL, "Could Not Unregister Class.", "SHUTDOWN ERROR", MB_OK | MB_ICONINFORMATION);

		// Set hInstance To NULL
		hInstance = NULL;
	}

}

//--------------------------------------------------------------------------
//	This Code Creates Our OpenGL Window.  Parameters Are
//	title			- Title To Appear At The Top Of The Window
//	width			- Width Of The GL Window Or Fullscreen Mode
//	height			- Height Of The GL Window Or Fullscreen Mode
//	bits			- Number Of Bits To Use For Color (8/16/24/32)
//	fullscreenflag	- Use Fullscreen Mode (TRUE) Or Windowed Mode (FALSE)
//--------------------------------------------------------------------------
BOOL CreateGLWindow(char * title, int width, int height, int bits, BOOL fullscreenflag)
{
	GLuint		PixelFormat;			// Holds The Results After Searching For A Match
	WNDCLASS	wc;						// Windows Class Structure
	DWORD		dwExStyle;				// Window Extended Style
	DWORD		dwStyle;				// Window Style
	RECT		rc;						// Grabs Rectangle Upper Left / Lower Right Values
	rc.left   = 0;						// Set Left Value To 0
	rc.right  = width;					// Set Right Value To Requested Width
	rc.top    = 0;						// Set Top Value To 0
	rc.bottom = height;					// Set Bottom Value To Requested Height

	// Set The Global Fullscreen Flag
	bFullScreen = fullscreenflag;

	hInstance			= GetModuleHandle(NULL);				// Grab An Instance For Our Window
	wc.style			= CS_HREDRAW | CS_VREDRAW | CS_OWNDC;	// Redraw On Size, And Own DC For Window.
	wc.lpfnWndProc		= (WNDPROC) WndProc;					// WndProc Handles Messages
	wc.cbClsExtra		= 0;									// No Extra Window Data
	wc.cbWndExtra		= 0;									// No Extra Window Data
	wc.hInstance		= hInstance;							// Set The Instance
	wc.hIcon			= LoadIcon(NULL, IDI_WINLOGO);			// Load The Default Icon
	wc.hCursor			= LoadCursor(NULL, IDC_ARROW);			// Load The Arrow Pointer
	wc.hbrBackground	= NULL;									// No Background Required For GL
	wc.lpszMenuName		= NULL;									// We Don't Want A Menu
	wc.lpszClassName	= "OpenGL";								// Set The Class Name

	// Attempt To Register The Window Class
	if (!RegisterClass(&wc))
	{
		MessageBox(NULL, "Failed To Register The Window Class.", "ERROR", MB_OK | MB_ICONEXCLAMATION);
		return GL_FAIL;
	}

	// Attempt Fullscreen Mode ?
	if (bFullScreen)
	{
		DEVMODE dmScreenSettings;								// Device Mode
		memset(&dmScreenSettings, 0, sizeof(dmScreenSettings));	// Makes Sure Memory's Cleared
		dmScreenSettings.dmSize = sizeof(dmScreenSettings);		// Size Of The Devmode Structure
		dmScreenSettings.dmPelsWidth	= width;				// Selected Screen Width
		dmScreenSettings.dmPelsHeight	= height;				// Selected Screen Height
		dmScreenSettings.dmBitsPerPel	= bits;					// Selected Bits Per Pixel
		dmScreenSettings.dmFields = DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT;

		// Try To Set Selected Mode And Get Results.  NOTE: CDS_FULLSCREEN Gets Rid Of Start Bar.
		if (ChangeDisplaySettings(&dmScreenSettings, CDS_FULLSCREEN) != DISP_CHANGE_SUCCESSFUL)
		{
			// If The Mode Fails, Offer Two Options.  Quit Or Use Windowed Mode.
			if (MessageBox(NULL, "The Requested Fullscreen Mode Is Not Supported By\nYour Video Card. Use Windowed Mode Instead?", "NeHe GL", MB_YESNO | MB_ICONEXCLAMATION) == IDYES)
			{
				bFullScreen = FALSE;		// Windowed Mode Selected.  Fullscreen = FALSE
			}
			else
			{
				// Pop Up A Message Box Letting User Know The Program Is Closing.
				MessageBox(NULL, "Program Will Now Close.", "ERROR", MB_OK | MB_ICONSTOP);
				return GL_FAIL;
			}
		}
	}

	// Are We Still In Fullscreen Mode ?
	if (bFullScreen)
	{
		dwExStyle = WS_EX_APPWINDOW;	// Window Extended Style
		dwStyle   = WS_POPUP;			// Windows Style
		ShowCursor(FALSE);				// Hide Mouse Pointer
	}
	else
	{
		dwExStyle = WS_EX_APPWINDOW | WS_EX_WINDOWEDGE;			// Window Extended Style
		dwStyle   = WS_OVERLAPPEDWINDOW;						// Windows Style
	}

	// Adjust Window To True Requested Size
	AdjustWindowRectEx(&rc, dwStyle, FALSE, dwExStyle);

	// Create The Window
	if (!(hWnd=CreateWindowEx(dwExStyle, "OpenGL", title,
		dwStyle | WS_CLIPSIBLINGS |	WS_CLIPCHILDREN,
		0, 0,
		rc.right - rc.left,
		rc.bottom - rc.top,
		NULL, NULL, hInstance, NULL))){

		// Reset The Display
		KillGLWindow();

		MessageBox(NULL, "Window Creation Error.", "ERROR", MB_OK | MB_ICONEXCLAMATION);

		return GL_FAIL;
	}

	static	PIXELFORMATDESCRIPTOR pfd =		// pfd Tells Windows How We Want Things To Be
	{
		sizeof(PIXELFORMATDESCRIPTOR),		// Size Of This Pixel Format Descriptor
		1,									// Version Number
		PFD_DRAW_TO_WINDOW |				// Format Must Support Window
		PFD_SUPPORT_OPENGL |				// Format Must Support OpenGL
		PFD_DOUBLEBUFFER,					// Must Support Double Buffering
		PFD_TYPE_RGBA,						// Request An RGBA Format
		0,									// Select Our Color Depth
		0, 0, 0, 0, 0, 0,					// Color Bits set below
		0,									// No Alpha Buffer
		0,									// Shift Bit Ignored
		0,									// No Accumulation Buffer
		0, 0, 0, 0,							// Accumulation Bits Ignored
		16,									// 16Bit Z-Buffer (Depth Buffer)
		0,									// No Stencil Buffer
		0,									// No Auxiliary Buffer
		PFD_MAIN_PLANE,						// Main Drawing Layer
		0,									// Reserved
		0, 0, 0								// Layer Masks Ignored
	};

	// Select Our Color Depth
	pfd.cColorBits = bits;

	// Did We Get A Device Context ?
	if (!(hDC = GetDC(hWnd)))
	{
		// Reset The Display
		KillGLWindow();

		MessageBox(NULL, "Can't Create A GL Device Context.", "ERROR", MB_OK | MB_ICONEXCLAMATION);

		return GL_FAIL;
	}

	// Did Windows Find A Matching Pixel Format ?
	if (!(PixelFormat = ChoosePixelFormat(hDC, &pfd)))
	{
		// Reset The Display
		KillGLWindow();

		MessageBox(NULL, "Can't Find A Suitable PixelFormat.", "ERROR", MB_OK | MB_ICONEXCLAMATION);

		return GL_FAIL;
	}

	// Are We Able To Set The Pixel Format ?
	if(!SetPixelFormat(hDC, PixelFormat, &pfd))
	{
		// Reset The Display
		KillGLWindow();

		MessageBox(NULL, "Can't Set The PixelFormat.", "ERROR", MB_OK | MB_ICONEXCLAMATION);

		return GL_FAIL;
	}

	// Are We Able To Get A Rendering Context ?
	if (!(hRC=wglCreateContext(hDC)))
	{
		// Reset The Display
		KillGLWindow();

		MessageBox(NULL, "Can't Create A GL Rendering Context.", "ERROR", MB_OK | MB_ICONEXCLAMATION);

		return GL_FAIL;
	}

	// Try To Activate The Rendering Context
	if(!wglMakeCurrent(hDC, hRC))
	{
		// Reset The Display
		KillGLWindow();

		MessageBox(NULL, "Can't Activate The GL Rendering Context.", "ERROR", MB_OK | MB_ICONEXCLAMATION);

		return GL_FAIL;
	}

	ShowWindow(hWnd, SW_SHOW);						// Show The Window
	SetForegroundWindow(hWnd);						// Slightly Higher Priority
	SetFocus(hWnd);									// Sets Keyboard Focus To The Window
	ReSizeGLScene(width, height);					// Set Up Our Perspective GL Screen

	// Initialize Our Newly Created GL Window
	if (!InitGL())
	{
		// Reset The Display
		KillGLWindow();

		MessageBox(NULL, "Initialization Failed.", "ERROR", MB_OK | MB_ICONEXCLAMATION);

		return GL_FAIL;
	}

	return GL_OK;

}


// ***********************************************************************
LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{

	// Check For Windows Messages
	switch (uMsg)
	{
		// Watch For Window Activate Message
	case WM_ACTIVATE:
		{
			// Check Minimization State
			if (!HIWORD(wParam))
			{
				bActive = TRUE;			// Program Is Active
			}
			else
			{
				bActive = FALSE;			// Program Is No Longer Active
			}

			return 0;						// Return To The Message Loop
		}

		// Intercept System Commands
	case WM_SYSCOMMAND:
		{
			// Check System Calls
			switch (wParam)
			{
			case SC_SCREENSAVE:		// Screensaver Trying To Start ?

			case SC_MONITORPOWER:	// Monitor Trying To Enter Powersave ?


				return 0;			// Prevent From Happening
			}
			break;					// Exit
		}

		// Did We Receive A Close Message ?
	case WM_CLOSE:
		{
			PostQuitMessage(0);				// Send A Quit Message

			return 0;						// Jump Back
		}

		// Is A Key Being Held Down ?
	case WM_KEYDOWN:
		{
			keys[wParam] = TRUE;			// If So, Mark It As TRUE

			return 0;						// Jump Back
		}

		// Has A Key Been Released ?
	case WM_KEYUP:
		{
			keys[wParam] = FALSE;			// If So, Mark It As FALSE

			return 0;						// Jump Back
		}

		// Resize The OpenGL Window
	case WM_SIZE:
		{
			ReSizeGLScene(LOWORD(lParam), HIWORD(lParam));	// LoWord = Width, HiWord = Height

			return 0;										// Jump Back
		}
	}

	// Pass All Unhandled Messages To DefWindowProc
	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}
